@@ -1068,19 +1068,21 @@ def general_import_supply(sess, action, vals, args):
 
         dc_account = add_arg(args, "DC Account", vals, 10)
         msn = add_arg(args, "Meter Serial Number", vals, 11)
-        pc_code = add_arg(args, "Profile Class", vals, 12)
+        dno_code = add_arg(args, "DNO Code", vals, 12)
+        dno = Party.get_dno_by_code(sess, dno_code)
+        pc_code = add_arg(args, "Profile Class", vals, 13)
         pc = Pc.get_by_code(sess, parse_pc_code(pc_code))
-        mtc_code = add_arg(args, "Meter Timeswitch Class", vals, 13)
-        cop_code = add_arg(args, "CoP", vals, 14)
+        mtc_code = add_arg(args, "Meter Timeswitch Class", vals, 14)
+        cop_code = add_arg(args, "CoP", vals, 15)
         cop = Cop.get_by_code(sess, cop_code)
-        ssc_code = add_arg(args, "Standard Settlement Configuration", vals, 15)
+        ssc_code = add_arg(args, "Standard Settlement Configuration", vals, 16)
         ssc = Ssc.get_by_code(sess, ssc_code) if len(ssc_code) > 0 else None
-        properties_str = add_arg(args, "Properties", vals, 16)
+        properties_str = add_arg(args, "Properties", vals, 17)
         try:
             properties = loads(properties_str)
         except ZishException as e:
             raise BadRequest("Can't parse the properties field. " + str(e))
-        imp_mpan_core = add_arg(args, "Import MPAN Core", vals, 17)
+        imp_mpan_core = add_arg(args, "Import MPAN Core", vals, 18)
         if len(imp_mpan_core) == 0:
             imp_mpan_core = None
         else:
@@ -1092,8 +1094,8 @@ def general_import_supply(sess, action, vals, args):
             imp_supplier_account = None
             imp_sc = None
         else:
-            imp_llfc_code = add_arg(args, "Import LLFC", vals, 18)
-            imp_sc_str = add_arg(args, "Import Agreed Supply Capacity", vals, 19)
+            imp_llfc_code = add_arg(args, "Import LLFC", vals, 19)
+            imp_sc_str = add_arg(args, "Import Agreed Supply Capacity", vals, 20)
             try:
                 imp_sc = int(imp_sc_str)
             except ValueError as e:
@@ -1102,9 +1104,9 @@ def general_import_supply(sess, action, vals, args):
                 )
 
             imp_supplier_contract_name = add_arg(
-                args, "Import Supplier Contract", vals, 20
+                args, "Import Supplier Contract", vals, 21
             )
-            imp_supplier_account = add_arg(args, "Import Supplier Account", vals, 21)
+            imp_supplier_account = add_arg(args, "Import Supplier Account", vals, 22)
             imp_supplier_contract = Contract.get_supplier_by_name(
                 sess, imp_supplier_contract_name
             )
@@ -1115,15 +1117,15 @@ def general_import_supply(sess, action, vals, args):
         exp_mpan_core = None
         exp_supplier_account = None
         if len(vals) > 22:
-            exp_mpan_core = add_arg(args, "Export MPAN Core", vals, 22)
+            exp_mpan_core = add_arg(args, "Export MPAN Core", vals, 23)
             if len(exp_mpan_core) == 0:
                 exp_mpan_core = None
             else:
                 exp_mpan_core = parse_mpan_core(exp_mpan_core)
 
             if exp_mpan_core is not None:
-                exp_llfc_code = add_arg(args, "Export LLFC", vals, 23)
-                exp_sc_str = add_arg(args, "Export Agreed Supply Capacity", vals, 24)
+                exp_llfc_code = add_arg(args, "Export LLFC", vals, 24)
+                exp_sc_str = add_arg(args, "Export Agreed Supply Capacity", vals, 25)
                 try:
                     exp_sc = int(exp_sc_str)
                 except ValueError as e:
@@ -1134,13 +1136,13 @@ def general_import_supply(sess, action, vals, args):
                     )
 
                 exp_supplier_contract_name = add_arg(
-                    args, "Export Supplier Contract", vals, 25
+                    args, "Export Supplier Contract", vals, 26
                 )
                 exp_supplier_contract = Contract.get_supplier_by_name(
                     sess, exp_supplier_contract_name
                 )
                 exp_supplier_account = add_arg(
-                    args, "Export Supplier Account", vals, 26
+                    args, "Export Supplier Account", vals, 27
                 )
 
         supply = site.insert_e_supply(
@@ -1156,6 +1158,7 @@ def general_import_supply(sess, action, vals, args):
             dc_contract,
             dc_account,
             msn,
+            dno,
             pc,
             mtc_code,
             cop,
@@ -1522,71 +1525,67 @@ class GeneralImporter(threading.Thread):
         return fields
 
     def run(self):
-        sess = None
         try:
-            sess = Session()
-            reader = csv.reader(self.f)
-            hh_data = []
-            for idx, line in enumerate(reader):
-                self.args = []
-                self.line_number = idx + 1
+            with Session() as sess:
+                reader = csv.reader(self.f)
+                hh_data = []
+                for idx, line in enumerate(reader):
+                    self.args = []
+                    self.line_number = idx + 1
 
-                if len(line) == 0:
-                    continue
+                    if len(line) == 0:
+                        continue
 
-                if len(line) > 0 and line[0].startswith("#"):
-                    continue
+                    if len(line) > 0 and line[0].startswith("#"):
+                        continue
 
-                action = add_arg(self.args, "action", line, 0).lower()
-                if action not in ALLOWED_ACTIONS:
-                    raise BadRequest(
-                        f"The 'action' field must be one of {ALLOWED_ACTIONS}"
-                    )
-                typ = add_arg(self.args, "type", line, 1).lower()
-                vals = line[2:]
-                if typ == "hh_datum":
-                    if action == "insert":
-                        hh_data.append(
-                            {
-                                "mpan_core": parse_mpan_core(
-                                    add_arg(self.args, "MPAN Core", vals, 0)
-                                ),
-                                "start_date": parse_hh_start(
-                                    add_arg(self.args, "Start Date", vals, 1)
-                                ),
-                                "channel_type": parse_channel_type(
-                                    add_arg(self.args, "Channel Type", vals, 2)
-                                ),
-                                "value": Decimal(add_arg(self.args, "Value", vals, 3)),
-                                "status": add_arg(self.args, "Status", vals, 4),
-                            }
+                    action = add_arg(self.args, "action", line, 0).lower()
+                    if action not in ALLOWED_ACTIONS:
+                        raise BadRequest(
+                            f"The 'action' field must be one of {ALLOWED_ACTIONS}"
                         )
-                else:
-                    try:
-                        typ_func = typ_funcs[typ]
-                        typ_func(sess, action, vals, self.args)
-                    except KeyError:
-                        raise BadRequest(f"The type {typ} is not recognized.")
+                    typ = add_arg(self.args, "type", line, 1).lower()
+                    vals = line[2:]
+                    if typ == "hh_datum":
+                        if action == "insert":
+                            hh_data.append(
+                                {
+                                    "mpan_core": parse_mpan_core(
+                                        add_arg(self.args, "MPAN Core", vals, 0)
+                                    ),
+                                    "start_date": parse_hh_start(
+                                        add_arg(self.args, "Start Date", vals, 1)
+                                    ),
+                                    "channel_type": parse_channel_type(
+                                        add_arg(self.args, "Channel Type", vals, 2)
+                                    ),
+                                    "value": Decimal(
+                                        add_arg(self.args, "Value", vals, 3)
+                                    ),
+                                    "status": add_arg(self.args, "Status", vals, 4),
+                                }
+                            )
+                    else:
+                        try:
+                            typ_func = typ_funcs[typ]
+                            typ_func(sess, action, vals, self.args)
+                        except KeyError:
+                            raise BadRequest(f"The type {typ} is not recognized.")
 
-            HhDatum.insert(sess, hh_data)
-            sess.commit()
+                HhDatum.insert(sess, hh_data)
+                sess.commit()
         except BadRequest as e:
-            sess.rollback()
             try:
                 self.rd_lock.acquire()
                 self.error_message = e.description
             finally:
                 self.rd_lock.release()
         except BaseException:
-            sess.rollback()
             try:
                 self.rd_lock.acquire()
                 self.error_message = traceback.format_exc()
             finally:
                 self.rd_lock.release()
-        finally:
-            if sess is not None:
-                sess.close()
 
 
 def start_process(f):
