@@ -264,6 +264,7 @@ def test_init_hh_data(sess, mocker):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -432,6 +433,7 @@ def test_init_hh_data_export(sess, mocker):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -619,6 +621,7 @@ def test_SupplySource_init_hh(sess, mocker):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -802,11 +805,12 @@ def test_SupplySource_init_nhh(sess, mocker):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
         comm,
-        ssc,
+        ssc.code,
         energisation_status,
         {},
         None,
