@@ -326,6 +326,7 @@ def virtual_bill(ds):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -347,6 +348,8 @@ def virtual_bill(ds):
 
     editor = UserRole.insert(sess, "editor")
     user = User.insert(sess, "admin@example.com", "xxx", editor, None)
+    user_id = user.id
+    site_code = site.code
 
     sess.commit()
 
@@ -355,7 +358,7 @@ def virtual_bill(ds):
         "scenario_start_month": start_date.month,
         "scenario_duration": months,
         "by_hh": False,
-        "site_codes": [site.code],
+        "site_codes": [site_code],
         "mpan_cores": None,
     }
     base_name = ["monthly_duration"]
@@ -374,7 +377,7 @@ def virtual_bill(ds):
     content(
         scenario_props,
         base_name,
-        user.id,
+        user_id,
         compression,
         now,
         is_bill_check,
@@ -611,6 +614,7 @@ def virtual_bill(ds):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -632,7 +636,8 @@ def virtual_bill(ds):
 
     editor = UserRole.insert(sess, "editor")
     user = User.insert(sess, "admin@example.com", "xxx", editor, None)
-
+    user_id = user.id
+    site_code = site.code
     sess.commit()
 
     scenario_props = {
@@ -640,7 +645,7 @@ def virtual_bill(ds):
         "scenario_start_month": start_date.month,
         "scenario_duration": months,
         "by_hh": False,
-        "site_codes": [site.code],
+        "site_codes": [site_code],
         "mpan_cores": None,
     }
     base_name = ["monthly_duration"]
@@ -659,7 +664,7 @@ def virtual_bill(ds):
     content(
         scenario_props,
         base_name,
-        user.id,
+        user_id,
         compression,
         now,
         is_bill_check,
@@ -844,6 +849,7 @@ def virtual_bill(ds):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -882,6 +888,8 @@ def virtual_bill(ds):
 
     editor = UserRole.insert(sess, "editor")
     user = User.insert(sess, "admin@example.com", "xxx", editor, None)
+    user_id = user.id
+    site_code = site.code
 
     sess.commit()
 
@@ -890,7 +898,7 @@ def virtual_bill(ds):
         "scenario_start_month": start_date.month,
         "scenario_duration": months,
         "by_hh": False,
-        "site_codes": [site.code],
+        "site_codes": [site_code],
         "mpan_cores": None,
     }
     base_name = ["monthly_duration"]
@@ -909,7 +917,7 @@ def virtual_bill(ds):
     content(
         scenario_props,
         base_name,
-        user.id,
+        user_id,
         compression,
         now,
         is_bill_check,
@@ -1254,6 +1262,7 @@ def virtual_bill(ds):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -1292,6 +1301,8 @@ def virtual_bill(ds):
 
     editor = UserRole.insert(sess, "editor")
     user = User.insert(sess, "admin@example.com", "xxx", editor, None)
+    user_id = user.id
+    site_code = site.code
 
     sess.commit()
 
@@ -1300,7 +1311,7 @@ def virtual_bill(ds):
         "scenario_start_month": start_date.month,
         "scenario_duration": months,
         "by_hh": False,
-        "site_codes": [site.code],
+        "site_codes": [site_code],
         "mpan_cores": [imp_mpan_core],
     }
     base_name = ["monthly_duration"]
@@ -1319,7 +1330,7 @@ def virtual_bill(ds):
     content(
         scenario_props,
         base_name,
-        user.id,
+        user_id,
         compression,
         now,
         is_bill_check,
@@ -1573,6 +1584,7 @@ def virtual_bill(ds):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -1613,6 +1625,8 @@ def virtual_bill(ds):
 
     editor = UserRole.insert(sess, "editor")
     user = User.insert(sess, "admin@example.com", "xxx", editor, None)
+    site_1_code = site_1.code
+    user_id = user.id
 
     sess.commit()
 
@@ -1621,7 +1635,7 @@ def virtual_bill(ds):
         "scenario_start_month": start_date.month,
         "scenario_duration": months,
         "by_hh": False,
-        "site_codes": [site_1.code],
+        "site_codes": [site_1_code],
         "mpan_cores": None,
     }
     base_name = ["monthly_duration"]
@@ -1640,7 +1654,7 @@ def virtual_bill(ds):
     content(
         scenario_props,
         base_name,
-        user.id,
+        user_id,
         compression,
         now,
         is_bill_check,
@@ -1900,6 +1914,7 @@ def virtual_bill(ds):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -1939,6 +1954,8 @@ def virtual_bill(ds):
 
     editor = UserRole.insert(sess, "editor")
     user = User.insert(sess, "admin@example.com", "xxx", editor, None)
+    user_id = user.id
+    site_code = site.code
 
     sess.commit()
 
@@ -1947,7 +1964,7 @@ def virtual_bill(ds):
         "scenario_start_month": start_date.month,
         "scenario_duration": months,
         "by_hh": False,
-        "site_codes": [site.code],
+        "site_codes": [site_code],
         "mpan_cores": None,
     }
     base_name = ["monthly_duration"]
@@ -1966,7 +1983,7 @@ def virtual_bill(ds):
     content(
         scenario_props,
         base_name,
-        user.id,
+        user_id,
         compression,
         now,
         is_bill_check,
@@ -2450,6 +2467,7 @@ def virtual_bill(ds):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -2488,6 +2506,8 @@ def virtual_bill(ds):
 
     editor = UserRole.insert(sess, "editor")
     user = User.insert(sess, "admin@example.com", "xxx", editor, None)
+    user_id = user.id
+    site_code = site.code
 
     sess.commit()
 
@@ -2496,7 +2516,7 @@ def virtual_bill(ds):
         "scenario_start_month": start_date.month,
         "scenario_duration": months,
         "by_hh": False,
-        "site_codes": [site.code],
+        "site_codes": [site_code],
         "mpan_cores": None,
     }
     base_name = ["monthly_duration"]
@@ -2515,7 +2535,7 @@ def virtual_bill(ds):
     content(
         scenario_props,
         base_name,
-        user.id,
+        user_id,
         compression,
         now,
         is_bill_check,
@@ -2862,6 +2882,7 @@ def virtual_bill(ds):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -2900,6 +2921,8 @@ def virtual_bill(ds):
 
     editor = UserRole.insert(sess, "editor")
     user = User.insert(sess, "admin@example.com", "xxx", editor, None)
+    user_id = user.id
+    site_code = site.code
 
     sess.commit()
 
@@ -2908,7 +2931,7 @@ def virtual_bill(ds):
         "scenario_start_month": start_date.month,
         "scenario_duration": months,
         "by_hh": False,
-        "site_codes": [site.code],
+        "site_codes": [site_code],
         "mpan_cores": None,
     }
     base_name = ["monthly_duration"]
@@ -2927,7 +2950,7 @@ def virtual_bill(ds):
     content(
         scenario_props,
         base_name,
-        user.id,
+        user_id,
         compression,
         now,
         is_bill_check,
@@ -3201,6 +3224,7 @@ def displaced_virtual_bill(ds):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -3232,6 +3256,7 @@ def displaced_virtual_bill(ds):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -3253,6 +3278,8 @@ def displaced_virtual_bill(ds):
 
     editor = UserRole.insert(sess, "editor")
     user = User.insert(sess, "admin@example.com", "xxx", editor, None)
+    user_id = user.id
+    site_code = site.code
 
     sess.commit()
 
@@ -3261,7 +3288,7 @@ def displaced_virtual_bill(ds):
         "scenario_start_month": start_date.month,
         "scenario_duration": months,
         "by_hh": False,
-        "site_codes": [site.code],
+        "site_codes": [site_code],
         "mpan_cores": None,
     }
     base_name = ["monthly_duration"]
@@ -3280,7 +3307,7 @@ def displaced_virtual_bill(ds):
     content(
         scenario_props,
         base_name,
-        user.id,
+        user_id,
         compression,
         now,
         is_bill_check,
@@ -3559,13 +3586,15 @@ def test_content_no_mpan_cores(mocker, sess, client):
     }
     base_name = ["no_mpan_cores"]
     user = User.get_by_email_address(sess, "admin@example.com")
+    user_id = user.id
+    sess.commit()
     compression = False
     now = to_utc(ct_datetime(2020, 1, 1))
     is_bill_check = False
     content(
         scenario_props,
         base_name,
-        user.id,
+        user_id,
         compression,
         now,
         is_bill_check,
