@@ -125,6 +125,7 @@ def test_process(mocker, sess):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -157,6 +158,7 @@ def test_process(mocker, sess):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -482,6 +484,7 @@ def test_process_in_chellow_not_ecoes(mocker, sess):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -706,11 +709,12 @@ def test_meter_type(mocker, sess):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
         comm,
-        ssc,
+        ssc.code,
         energisation_status,
         {},
         "22 7868 6232 789",
