@@ -27,6 +27,7 @@ from chellow.models import (
     MtcSsc,
     Participant,
     Pc,
+    ReadType,
     Scenario,
     Site,
     Source,
@@ -128,6 +129,7 @@ def test_Bill_update(sess):
         dc_contract,
         "ghyy3",
         "e1msn",
+        dno,
         pc,
         "845",
         cop,
@@ -151,7 +153,6 @@ def test_Bill_update(sess):
     bill_type = BillType.get_by_code(sess, "N")
 
     sess.commit()
-    sess.expire_all()
 
     batch.insert_bill(
         sess,
@@ -170,6 +171,135 @@ def test_Bill_update(sess):
     )
 
 
+def test_Bill_insert_read_duplicate(sess):
+    vf = to_utc(ct_datetime(1996, 1, 1))
+    site = Site.insert(sess, "22488", "Water Works")
+    insert_sources(sess)
+    source = Source.get_by_code(sess, "net")
+    gsp_group = GspGroup.insert(sess, "_L", "South Western")
+    participant = Participant.insert(sess, "hhak", "AK Industries")
+    market_role_X = MarketRole.insert(sess, "X", "Supplier")
+    market_role_M = MarketRole.insert(sess, "M", "Mop")
+    market_role_C = MarketRole.insert(sess, "C", "HH Dc")
+    market_role_R = MarketRole.insert(sess, "R", "Distributor")
+    participant.insert_party(sess, market_role_M, "Fusion Mop Ltd", vf, None, None)
+    participant.insert_party(sess, market_role_X, "Fusion Ltc", vf, None, None)
+    participant.insert_party(sess, market_role_C, "Fusion DC", vf, None, None)
+    mop_contract = Contract.insert_mop(
+        sess, "Fusion", participant, "", {}, vf, None, {}
+    )
+    dc_contract = Contract.insert_dc(
+        sess, "Fusion DC 2000", participant, "", {}, vf, None, {}
+    )
+    pc = Pc.insert(sess, "00", "hh", vf, None)
+    insert_cops(sess)
+    cop = Cop.get_by_code(sess, "5")
+    insert_comms(sess)
+    comm = Comm.get_by_code(sess, "GSM")
+    imp_supplier_contract = Contract.insert_supplier(
+        sess, "Fusion Supplier 2000", participant, "", {}, vf, None, {}
+    )
+    dno = participant.insert_party(sess, market_role_R, "WPD", vf, None, "22")
+    meter_type = MeterType.insert(sess, "C5", "COP 1-5", vf, None)
+    meter_payment_type = MeterPaymentType.insert(sess, "CR", "Credit", vf, None)
+    mtc = Mtc.insert(sess, "845", False, True, vf, None)
+    mtc_participant = MtcParticipant.insert(
+        sess,
+        mtc,
+        participant,
+        "HH COP5 And Above With Comms",
+        False,
+        True,
+        meter_type,
+        meter_payment_type,
+        0,
+        vf,
+        None,
+    )
+    insert_voltage_levels(sess)
+    voltage_level = VoltageLevel.get_by_code(sess, "HV")
+    llfc = dno.insert_llfc(
+        sess, "510", "PC 5-8 & HH HV", voltage_level, False, True, vf, None
+    )
+    MtcLlfc.insert(sess, mtc_participant, llfc, vf, None)
+    insert_energisation_statuses(sess)
+    energisation_status = EnergisationStatus.get_by_code(sess, "E")
+    supply = site.insert_e_supply(
+        sess,
+        source,
+        None,
+        "Bob",
+        utc_datetime(2020, 1, 1),
+        utc_datetime(2020, 1, 31),
+        gsp_group,
+        mop_contract,
+        "773",
+        dc_contract,
+        "ghyy3",
+        "hgjeyhuw",
+        dno,
+        pc,
+        "845",
+        cop,
+        comm,
+        None,
+        energisation_status,
+        {},
+        "22 7867 6232 781",
+        "510",
+        imp_supplier_contract,
+        "7748",
+        361,
+        None,
+        None,
+        None,
+        None,
+        None,
+    )
+    batch = imp_supplier_contract.insert_batch(sess, "b1", "batch 1")
+    insert_bill_types(sess)
+    bill_type_N = BillType.get_by_code(sess, "N")
+    bill = batch.insert_bill(
+        sess,
+        "hrghj88",
+        "74hjkgjk",
+        to_utc(ct_datetime(2020, 2, 10)),
+        to_utc(ct_datetime(2020, 2, 2)),
+        to_utc(ct_datetime(2020, 3, 1)),
+        Decimal("0"),
+        Decimal("0.00"),
+        Decimal("0.00"),
+        Decimal("0.00"),
+        bill_type_N,
+        {},
+        supply,
+    )
+    tpr = Tpr.insert(sess, "00001", False, True)
+    read_type = ReadType.insert(sess, "A", "Actual")
+    sess.commit()
+
+    args = (
+        sess,
+        tpr,
+        1,
+        "kWh",
+        "xxx",
+        "2000",
+        to_utc(ct_datetime(2020, 2, 10)),
+        Decimal("100"),
+        read_type,
+        to_utc(ct_datetime(2020, 3, 10)),
+        Decimal("200"),
+        read_type,
+    )
+
+    bill.insert_read(*args)
+    with pytest.raises(BadRequest, match="Duplicate register reads aren't allowed"):
+        bill.insert_read(*args)
+
+    sess.rollback()
+
+
 def test_Era_update(sess):
     """Successful"""
     valid_from = to_utc(ct_datetime(1996, 1, 1))
@@ -240,6 +370,7 @@ def test_Era_update(sess):
         dc_contract,
         "ghyy3",
         "e1msn",
+        dno,
         pc,
         "845",
         cop,
@@ -375,11 +506,12 @@ def test_Era_meter_category_dumb(sess):
         dc_contract,
         "ghyy3",
         "e1msn",
+        dno,
         pc,
         "845",
         cop,
         comm,
-        ssc,
+        ssc.code,
         energisation_status,
         {},
         imp_mpan_core,
@@ -595,6 +727,7 @@ def test_Era_init_llfc_valid_to(sess):
             dc_contract,
             "ghyy3",
             "e1msn",
+            dno,
             pc,
             "845",
             cop,
@@ -814,6 +947,7 @@ def test_Supply_insert_era_at(sess):
         dc_contract,
         "ghyy3",
         era1_msn,
+        dno,
         pc,
         "845",
         cop,
@@ -871,6 +1005,118 @@ def test_Supply_insert_era_at(sess):
     assert era3.msn == era1.msn
 
 
+def test_Supply_insert_era_at_hh_data(sess):
+    """Check HH data is moved to new era"""
+    vf = to_utc(ct_datetime(1996, 1, 1))
+    site = Site.insert(sess, "CI017", "Water Works")
+    market_role_Z = MarketRole.insert(sess, "Z", "Non-core")
+    participant = Participant.insert(sess, "CALB", "AK Industries")
+    participant.insert_party(sess, market_role_Z, "None core", vf, None, None)
+    market_role_X = MarketRole.insert(sess, "X", "Supplier")
+    market_role_M = MarketRole.insert(sess, "M", "Mop")
+    market_role_C = MarketRole.insert(sess, "C", "HH Dc")
+    market_role_R = MarketRole.insert(sess, "R", "Distributor")
+    participant.insert_party(sess, market_role_M, "Fusion Mop", vf, None, None)
+    participant.insert_party(sess, market_role_X, "Fusion Ltc", vf, None, None)
+    participant.insert_party(sess, market_role_C, "Fusion DC", vf, None, None)
+    mop_contract = Contract.insert_mop(
+        sess, "Fusion", participant, "", {}, vf, None, {}
+    )
+    dc_contract = Contract.insert_dc(
+        sess, "Fusion DC 2000", participant, "", {}, vf, None, {}
+    )
+    pc = Pc.insert(sess, "00", "hh", vf, None)
+    insert_cops(sess)
+    cop = Cop.get_by_code(sess, "5")
+    insert_comms(sess)
+    comm = Comm.get_by_code(sess, "GSM")
+    imp_supplier_contract = Contract.insert_supplier(
+        sess,
+        "Fusion Supplier 2000",
+        participant,
+        "",
+        {},
+        vf,
+        None,
+        {},
+    )
+    dno = participant.insert_party(sess, market_role_R, "WPD", vf, None, "22")
+    meter_type = MeterType.insert(sess, "C5", "COP 1-5", vf, None)
+    meter_payment_type = MeterPaymentType.insert(sess, "CR", "Credit", vf, None)
+    mtc = Mtc.insert(sess, "845", False, True, vf, None)
+    mtc_participant = MtcParticipant.insert(
+        sess,
+        mtc,
+        participant,
+        "HH COP5 And Above With Comms",
+        False,
+        True,
+        meter_type,
+        meter_payment_type,
+        0,
+        vf,
+        None,
+    )
+    insert_voltage_levels(sess)
+    voltage_level = VoltageLevel.get_by_code(sess, "HV")
+    llfc = dno.insert_llfc(
+        sess, "510", "PC 5-8 & HH HV", voltage_level, False, True, vf, None
+    )
+    MtcLlfc.insert(sess, mtc_participant, llfc, vf, None)
+    insert_sources(sess)
+    source = Source.get_by_code(sess, "net")
+    insert_energisation_statuses(sess)
+    energisation_status = EnergisationStatus.get_by_code(sess, "E")
+    gsp_group = GspGroup.insert(sess, "_L", "South Western")
+    era1_msn = "e1msn"
+    imp_mpan_core = "22 7867 6232 781"
+    supply = site.insert_e_supply(
+        sess,
+        source,
+        None,
+        "Bob",
+        utc_datetime(2000, 1, 1),
+        None,
+        gsp_group,
+        mop_contract,
+        "773",
+        dc_contract,
+        "ghyy3",
+        era1_msn,
+        dno,
+        pc,
+        "845",
+        cop,
+        comm,
+        None,
+        energisation_status,
+        {},
+        imp_mpan_core,
+        "510",
+        imp_supplier_contract,
+        "7748",
+        361,
+        None,
+        None,
+        None,
+        None,
+        None,
+    )
+    sess.commit()
+
+    era1 = supply.eras[0]
+
+    channel = era1.insert_channel(sess, True, "ACTIVE")
+    raw_hh = {
+        "start_date": to_utc(ct_datetime(2010, 1, 1)),
+        "status": "A",
+        "value": Decimal("10"),
+    }
+    channel.add_hh_data(sess, [raw_hh])
+
+    supply.insert_era_at(sess, utc_datetime(2009, 7, 31, 23, 30))
+
+
 def test_Scenario_init(sess):
     name = "scenario_bau"
     properties = {
