@@ -122,151 +122,6 @@ def test_local_reports_post(client, sess):
     assert report.template == ""
 
 
-def test_site_edit_post_fail(client, sess):
-    site = Site.insert(sess, "CI017", "Water Works")
-
-    market_role_Z = MarketRole.get_by_code(sess, "Z")
-    participant = Participant.insert(sess, "CALB", "AK Industries")
-    participant.insert_party(
-        sess, market_role_Z, "None core", utc_datetime(2000, 1, 1), None, None
-    )
-    bank_holiday_rate_script = {"bank_holidays": []}
-    Contract.insert_non_core(
-        sess,
-        "bank_holidays",
-        "",
-        {},
-        utc_datetime(2000, 1, 1),
-        None,
-        bank_holiday_rate_script,
-    )
-    market_role_X = MarketRole.insert(sess, "X", "Supplier")
-    market_role_M = MarketRole.insert(sess, "M", "Mop")
-    market_role_C = MarketRole.insert(sess, "C", "HH Dc")
-    market_role_R = MarketRole.insert(sess, "R", "Distributor")
-    participant.insert_party(
-        sess, market_role_M, "Fusion Mop Ltd", utc_datetime(2000, 1, 1), None, None
-    )
-    participant.insert_party(
-        sess, market_role_X, "Fusion Ltc", utc_datetime(2000, 1, 1), None, None
-    )
-    participant.insert_party(
-        sess, market_role_C, "Fusion DC", utc_datetime(2000, 1, 1), None, None
-    )
-    mop_contract = Contract.insert_mop(
-        sess, "Fusion", participant, "", {}, utc_datetime(2000, 1, 1), None, {}
-    )
-    dc_contract = Contract.insert_dc(
-        sess, "Fusion DC 2000", participant, "", {}, utc_datetime(2000, 1, 1), None, {}
-    )
-    pc = Pc.insert(sess, "00", "hh", utc_datetime(2000, 1, 1), None)
-    insert_cops(sess)
-    insert_comms(sess)
-    cop = Cop.get_by_code(sess, "5")
-    imp_supplier_contract = Contract.insert_supplier(
-        sess,
-        "Fusion Supplier 2000",
-        participant,
-        "",
-        {},
-        utc_datetime(2000, 1, 1),
-        None,
-        {},
-    )
-    dno = participant.insert_party(
-        sess, market_role_R, "WPD", utc_datetime(2000, 1, 1), None, "22"
-    )
-    meter_type = MeterType.insert(sess, "C5", "COP 1-5", utc_datetime(2000, 1, 1), None)
-    meter_payment_type = MeterPaymentType.insert(
-        sess, "CR", "Credit", utc_datetime(1996, 1, 1), None
-    )
-    mtc = Mtc.insert(
-        sess,
-        "845",
-        False,
-        True,
-        utc_datetime(1996, 1, 1),
-        None,
-    )
-    MtcParticipant.insert(
-        sess,
-        mtc,
-        participant,
-        "HH COP5 And Above With Comms",
-        False,
-        True,
-        meter_type,
-        meter_payment_type,
-        0,
-        utc_datetime(1996, 1, 1),
-        None,
-    )
-    insert_voltage_levels(sess)
-    voltage_level = VoltageLevel.get_by_code(sess, "HV")
-    dno.insert_llfc(
-        sess,
-        "510",
-        "PC 5-8 & HH HV",
-        voltage_level,
-        False,
-        True,
-        utc_datetime(1996, 1, 1),
-        None,
-    )
-    dno.insert_llfc(
-        sess,
-        "521",
-        "Export (HV)",
-        voltage_level,
-        False,
-        False,
-        utc_datetime(1996, 1, 1),
-        None,
-    )
-    insert_sources(sess)
-    source = Source.get_by_code(sess, "net")
-    insert_energisation_statuses(sess)
-    energisation_status = EnergisationStatus.get_by_code(sess, "E")
-    gsp_group = GspGroup.insert(sess, "_L", "South Western")
-    sess.commit()
-
-    data = {
-        "source_id": str(source.id),
-        "name": "Bob",
-        "start_date_year": "2000",
-        "start_date_month": "01",
-        "start_date_day": "01",
-        "start_date_hour": "00",
-        "start_date_minute": "00",
-        "insert_electricity": "Insert Electricity",
-        "mop_contract_id": str(mop_contract.id),
-        "dc_contract_id": str(dc_contract.id),
-        "pc_id": str(pc.id),
-        "cop_id": str(cop.id),
-        "imp_supplier_contract_id": str(imp_supplier_contract.id),
-        "energisation_status_id": str(energisation_status.id),
-        "gsp_group_id": str(gsp_group.id),
-    }
-    response = client.post(f"/sites/{site.id}/edit", data=data)
-
-    patterns = [
-        r'<select name="energisation_status_id">\s*'
-        r'<option value="2">D - De-Energised</option>\s*'
-        r'<option value="1" selected>E - Energised</option>\s*'
-        r"</select>",
-        r'<select name="comm_id">\s*'
-        r'<option value="3">GPRS General Packet Radio Service</option>\s*'
-        r'<option value="2">GSM Global System for Mobile Communications</option>\s*'
-        r'<option value="6">HP Handheld Perm</option>\s*'
-        r'<option value="1">IP Internet Protocol</option>\s*'
-        r'<option value="5">PK Paknet</option>\s*'
-        r'<option value="4">PS Public Switch \(BT Line\)</option>\s*'
-        r'<option value="7">SUB Sub Meter</option>\s*'
-        r"</select>",
-    ]
-    match(response, 400, *patterns)
-
-
 def test_site_get(client, sess):
     valid_from = to_utc(ct_datetime(2000, 1, 1))
     site = Site.insert(sess, "CI017", "Water Works")
@@ -379,6 +234,7 @@ def test_site_get(client, sess):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -521,11 +377,12 @@ def test_site_get_dumb(client, sess):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
         comm,
-        ssc,
+        ssc.code,
         energisation_status,
         {},
         "22 0470 7514 535",
@@ -674,11 +531,12 @@ def test_supplies_get_e(sess, client):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
         comm,
-        ssc,
+        ssc.code,
         energisation_status,
         {},
         "22 0470 7514 535",
@@ -705,11 +563,12 @@ def test_supplies_get_e(sess, client):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
         comm,
-        ssc,
+        ssc.code,
         energisation_status,
         {},
         "22 0471 7514 532",
@@ -897,6 +756,7 @@ def test_general_import_post_full(sess, client):
         dc_contract,
         "ghyy3",
         "hgjeyhuw",
+        dno,
         pc,
         "845",
         cop,
@@ -1043,57 +903,15 @@ def test_non_core_rate_script_edit_get(sess, client):
     match(response, 200)
 
 
-def test_site_edit_get(sess, client, app):
+def test_site_edit_get(sess, client):
     vf = to_utc(ct_datetime(2000, 1, 1))
     site = Site.insert(sess, "CI017", "Water Works")
 
-    market_role_Z = MarketRole.get_by_code(sess, "Z")
-    participant = Participant.insert(sess, "CALB", "AK Industries")
-    participant.insert_party(sess, market_role_Z, "None core", vf, None, None)
-    market_role_X = MarketRole.insert(sess, "X", "Supplier")
-    market_role_M = MarketRole.insert(sess, "M", "Mop")
-    market_role_C = MarketRole.insert(sess, "C", "HH Dc")
-    market_role_R = MarketRole.insert(sess, "R", "Distributor")
-    participant.insert_party(sess, market_role_M, "Fusion Mop Ltd", vf, None, None)
-    participant.insert_party(sess, market_role_X, "Fusion Ltc", vf, None, None)
-    participant.insert_party(sess, market_role_C, "Fusion DC", vf, None, None)
-    Contract.insert_mop(sess, "Fusion", participant, "", {}, vf, None, {})
-    Contract.insert_dc(sess, "Fusion DC 2000", participant, "", {}, vf, None, {})
-    insert_cops(sess)
-    insert_comms(sess)
-    Contract.insert_supplier(
-        sess, "Fusion Supplier 2000", participant, "", {}, vf, None, {}
-    )
-    dno = participant.insert_party(sess, market_role_R, "WPD", vf, None, "22")
-    meter_type = MeterType.insert(sess, "C5", "COP 1-5", utc_datetime(2000, 1, 1), None)
-    meter_payment_type = MeterPaymentType.insert(sess, "CR", "Credit", vf, None)
-    mtc = Mtc.insert(sess, "845", False, True, vf, None)
-    MtcParticipant.insert(
-        sess,
-        mtc,
-        participant,
-        "HH COP5 And Above With Comms",
-        False,
-        True,
-        meter_type,
-        meter_payment_type,
-        0,
-        vf,
-        None,
-    )
-    insert_voltage_levels(sess)
-    voltage_level = VoltageLevel.get_by_code(sess, "HV")
-    dno.insert_llfc(sess, "510", "PC 5-8 & HH HV", voltage_level, False, True, vf, None)
-    insert_sources(sess)
-    insert_energisation_statuses(sess)
-
     GContract.insert_industry(sess, "cv", "", {}, vf, None, {})
     sess.commit()
 
     response = client.get(f"/sites/{site.id}/edit")
     patterns = [
-        r'<select name="comm_id">\s*'
-        r'<option value="3">GPRS General Packet Radio Service</option>\s*',
         r'<select name="g_contract_id">\s*</select>',
     ]
     match(response, 200, *patterns)
@@ -1104,242 +922,6 @@ def test_rate_server_get(sess, client):
     match(response, 200)
 
 
-def test_site_edit_post_hh(sess, client):
-    valid_from = to_utc(ct_datetime(1996, 1, 1))
-    site = Site.insert(sess, "CI017", "Water Works")
-
-    market_role_Z = MarketRole.get_by_code(sess, "Z")
-    participant = Participant.insert(sess, "CALB", "AK Industries")
-    participant.insert_party(
-        sess, market_role_Z, "None core", utc_datetime(2000, 1, 1), None, None
-    )
-    market_role_X = MarketRole.insert(sess, "X", "Supplier")
-    market_role_M = MarketRole.insert(sess, "M", "Mop")
-    market_role_C = MarketRole.insert(sess, "C", "HH Dc")
-    market_role_R = MarketRole.insert(sess, "R", "Distributor")
-    participant.insert_party(
-        sess, market_role_M, "Fusion Mop Ltd", utc_datetime(2000, 1, 1), None, None
-    )
-    participant.insert_party(
-        sess, market_role_X, "Fusion Ltc", utc_datetime(2000, 1, 1), None, None
-    )
-    participant.insert_party(
-        sess, market_role_C, "Fusion DC", utc_datetime(2000, 1, 1), None, None
-    )
-    mop_contract = Contract.insert_mop(
-        sess, "Fusion", participant, "", {}, utc_datetime(2000, 1, 1), None, {}
-    )
-    dc_contract = Contract.insert_dc(
-        sess, "Fusion DC 2000", participant, "", {}, utc_datetime(2000, 1, 1), None, {}
-    )
-    pc = Pc.insert(sess, "00", "hh", utc_datetime(2000, 1, 1), None)
-    insert_cops(sess)
-    cop = Cop.get_by_code(sess, "5")
-    insert_comms(sess)
-    comm = Comm.get_by_code(sess, "GSM")
-    imp_supplier_contract = Contract.insert_supplier(
-        sess,
-        "Fusion Supplier 2000",
-        participant,
-        "",
-        {},
-        utc_datetime(2000, 1, 1),
-        None,
-        {},
-    )
-    dno = participant.insert_party(
-        sess, market_role_R, "WPD", utc_datetime(2000, 1, 1), None, "22"
-    )
-    meter_type = MeterType.insert(sess, "C5", "COP 1-5", utc_datetime(2000, 1, 1), None)
-    meter_payment_type = MeterPaymentType.insert(
-        sess, "CR", "Credit", utc_datetime(1996, 1, 1), None
-    )
-    mtc = Mtc.insert(sess, "845", False, True, valid_from, None)
-    mtc_participant = MtcParticipant.insert(
-        sess,
-        mtc,
-        participant,
-        "HH COP5 And Above With Comms",
-        False,
-        True,
-        meter_type,
-        meter_payment_type,
-        0,
-        utc_datetime(1996, 1, 1),
-        None,
-    )
-    insert_voltage_levels(sess)
-    voltage_level = VoltageLevel.get_by_code(sess, "HV")
-    llfc = dno.insert_llfc(
-        sess,
-        "510",
-        "PC 5-8 & HH HV",
-        voltage_level,
-        False,
-        True,
-        utc_datetime(1996, 1, 1),
-        None,
-    )
-    MtcLlfc.insert(sess, mtc_participant, llfc, valid_from, None)
-    insert_sources(sess)
-    source = Source.get_by_code(sess, "net")
-    gsp_group = GspGroup.insert(sess, "_L", "South Western")
-    insert_energisation_statuses(sess)
-    energisation_status = EnergisationStatus.get_by_code(sess, "E")
-    sess.commit()
-
-    data = {
-        "source_id": source.id,
-        "gsp_group_id": gsp_group.id,
-        "mop_contract_id": mop_contract.id,
-        "dc_contract_id": dc_contract.id,
-        "dc_account": "dc1",
-        "msn": "jjl4",
-        "pc_id": pc.id,
-        "mtc_code": "845",
-        "cop_id": cop.id,
-        "comm_id": comm.id,
-        "ssc_code": "",
-        "properties": "{}",
-        "start_year": "2021",
-        "start_month": "01",
-        "start_day": "01",
-        "start_hour": "00",
-        "start_minute": "00",
-        "mop_account": "ma1",
-        "imp_mpan_core": "22 6644 1123 880",
-        "imp_supplier_contract_id": imp_supplier_contract.id,
-        "imp_supplier_account": "sup1",
-        "imp_sc": "200",
-        "imp_llfc_code": "510",
-        "energisation_status_id": energisation_status.id,
-        "insert_electricity": "insert_electricity",
-        "name": "main",
-    }
-    response = client.post(f"/sites/{site.id}/edit", data=data)
-    match(response, 303, "/e/supplies")
-
-
-def test_site_edit_post_nhh(sess, client):
-    valid_from = to_utc(ct_datetime(1996, 1, 1))
-    site = Site.insert(sess, "CI017", "Water Works")
-
-    market_role_Z = MarketRole.get_by_code(sess, "Z")
-    participant = Participant.insert(sess, "CALB", "AK Industries")
-    participant.insert_party(
-        sess, market_role_Z, "None core", utc_datetime(2000, 1, 1), None, None
-    )
-    market_role_X = MarketRole.insert(sess, "X", "Supplier")
-    market_role_M = MarketRole.insert(sess, "M", "Mop")
-    market_role_C = MarketRole.insert(sess, "C", "HH Dc")
-    market_role_R = MarketRole.insert(sess, "R", "Distributor")
-    participant.insert_party(
-        sess, market_role_M, "Fusion Mop Ltd", utc_datetime(2000, 1, 1), None, None
-    )
-    participant.insert_party(
-        sess, market_role_X, "Fusion Ltc", utc_datetime(2000, 1, 1), None, None
-    )
-    participant.insert_party(
-        sess, market_role_C, "Fusion DC", utc_datetime(2000, 1, 1), None, None
-    )
-    mop_contract = Contract.insert_mop(
-        sess, "Fusion", participant, "", {}, utc_datetime(2000, 1, 1), None, {}
-    )
-    dc_contract = Contract.insert_dc(
-        sess, "Fusion DC 2000", participant, "", {}, utc_datetime(2000, 1, 1), None, {}
-    )
-    pc = Pc.insert(sess, "03", "hh", utc_datetime(2000, 1, 1), None)
-    insert_cops(sess)
-    cop = Cop.get_by_code(sess, "5")
-    insert_comms(sess)
-    comm = Comm.get_by_code(sess, "GSM")
-    imp_supplier_contract = Contract.insert_supplier(
-        sess,
-        "Fusion Supplier 2000",
-        participant,
-        "",
-        {},
-        utc_datetime(2000, 1, 1),
-        None,
-        {},
-    )
-    dno = participant.insert_party(
-        sess, market_role_R, "WPD", utc_datetime(2000, 1, 1), None, "22"
-    )
-    meter_type = MeterType.insert(sess, "C5", "COP 1-5", utc_datetime(2000, 1, 1), None)
-    meter_payment_type = MeterPaymentType.insert(
-        sess, "CR", "Credit", utc_datetime(1996, 1, 1), None
-    )
-    mtc = Mtc.insert(sess, "845", False, True, valid_from, None)
-    mtc_participant = MtcParticipant.insert(
-        sess,
-        mtc,
-        participant,
-        "HH COP5 And Above With Comms",
-        False,
-        True,
-        meter_type,
-        meter_payment_type,
-        0,
-        valid_from,
-        None,
-    )
-    insert_voltage_levels(sess)
-    voltage_level = VoltageLevel.get_by_code(sess, "HV")
-    llfc = dno.insert_llfc(
-        sess,
-        "510",
-        "PC 5-8 & HH HV",
-        voltage_level,
-        False,
-        True,
-        utc_datetime(1996, 1, 1),
-        None,
-    )
-    insert_sources(sess)
-    source = Source.get_by_code(sess, "net")
-    gsp_group = GspGroup.insert(sess, "_L", "South Western")
-    insert_energisation_statuses(sess)
-    energisation_status = EnergisationStatus.get_by_code(sess, "E")
-    ssc = Ssc.insert(sess, "0001", "All", True, utc_datetime(1996, 1, 1), None)
-    MtcLlfc.insert(sess, mtc_participant, llfc, valid_from, None)
-    mtc_ssc = MtcSsc.insert(sess, mtc_participant, ssc, valid_from, None)
-    mtc_llfc_ssc = MtcLlfcSsc.insert(sess, mtc_ssc, llfc, valid_from, None)
-    MtcLlfcSscPc.insert(sess, mtc_llfc_ssc, pc, valid_from, None)
-    sess.commit()
-
-    data = {
-        "source_id": source.id,
-        "gsp_group_id": gsp_group.id,
-        "mop_contract_id": mop_contract.id,
-        "dc_contract_id": dc_contract.id,
-        "dc_account": "dc1",
-        "msn": "jjl4",
-        "pc_id": pc.id,
-        "mtc_code": "845",
-        "cop_id": cop.id,
-        "comm_id": comm.id,
-        "ssc_code": ssc.code,
-        "properties": "{}",
-        "start_year": "2021",
-        "start_month": "01",
-        "start_day": "01",
-        "start_hour": "00",
-        "start_minute": "00",
-        "mop_account": "ma1",
-        "imp_mpan_core": "22 6644 1123 880",
-        "imp_supplier_contract_id": imp_supplier_contract.id,
-        "imp_supplier_account": "sup1",
-        "imp_sc": "200",
-        "imp_llfc_code": "510",
-        "energisation_status_id": energisation_status.id,
-        "insert_electricity": "insert_electricity",
-        "name": "main",
-    }
-    response = client.post(f"/sites/{site.id}/edit", data=data)
-    match(response, 303)
-
-
 def test_report_run_spreadsheet_get(sess, client):
     report_run = ReportRun.insert(sess, "bill_check", None, "_b_88", {})
     report_run.insert_row(sess, "", ["clump"], {}, {})
